#include "math.h"
#include "F16_AeroData_cfnc.c"              /* LOFI Look-up header file*/

#define pi 3.14159
#define g    32.17   // gravity, ft/s^2 
#define m    636.94  // mass, slugs 
#define B    30.0    // span, ft 
#define S    300.0   // planform area, ft^2 
#define cbar 11.32   // mean aero chord, ft 
#define xcgr 0.35    // reference center of gravity as a fraction of cbar 
#define xcg  0.30    // center of gravity as a fraction of cbar. 
#define Heng 0.0     // turbine momentum along roll axis. 
#define el_bw  20.0  // elevator bandwidth, correct value is 20.2Hz

#define Jy   55814.0  // slug-ft^2  
#define Jxz  982.0    // slug-ft^2      
#define Jz   63100.0  // slug-ft^2 
#define Jx   9496.0   // slug-ft^2 

void aerodata_altitude(double *xu, double *f, void *user_data){
    
//     DifferentialState x_theta x_Q x_alpha Vt;
//     Control u_el u_ft;
    
    double *temp;
    double beta = 0.0;      // only longitudinal dynamics considered
    double r2d  = 180.0/pi; // radians to degrees 
  
    double t = xu[0];       // time
    double x_alt = xu[1];   // pitch (state)
    double x_Q = xu[2];     // pitch rate (state)
    double x_U = xu[3];     // U, body velocity in x (state) 
    double x_W = xu[4];     // W, body velocity in z (state)
    double u_theta = xu[5]; // pitch reference (input to pitch controller)    
    double p_ft = xu[6];    // thrust (parameter), from a speed controller 
    double p_qbar = xu[7];  // dynamic pressure (parameter)
    
    // intermediate states 
    double Vt = 500.0;  // initial value
    double Czq;
    double Cz;       
    double dZdQ;       
    double Cz_tot;       
    double Cxq;
    double Cx;
    double dXdQ;
    double Cx_tot;
    double X_tot;       
    double Z_tot;

    // temporary memory allocation
    temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

    /* %%%%%%%%%%%%%%%%%%
          Table lookup
       %%%%%%%%%%%%%%%%%% */    

    damping(x_alpha*r2d,temp);
        Cxq = temp[0];
        //Cyr = temp[1];
        //Cyp = temp[2];
        Czq = temp[3];
        //Clr = temp[4];
        //Clp = temp[5];
        Cmq = temp[6];
        //Cnr = temp[7];
        //Cnp = temp[8];

    cxcm(x_alpha*r2d,u_el,temp);
        Cx = temp[0];
        Cm = temp[1];

    cz(x_alpha*r2d,beta,u_el,temp);
        Cz = temp[0];
                            
    free(temp); 
        
    // Intermediate states
        
    // Cz_tot
    dZdQ = (cbar/(2*Vt))*Czq;
    Cz_tot = Cz + dZdQ*x_Q;
  
    // Cx_tot
    dXdQ = (cbar/(2*Vt))*Cxq;
    Cx_tot = Cx + dXdQ*x_Q; 

    // total longitudinal forces from coefficients, X_tot and Z_tot
    X_tot = p_qbar*S*Cx_tot;
    Z_tot = p_qbar*S*Cz_tot;    
    
    // total longitudinal speed
    Vt = sqrt(pow(U,2)*pow(W,2));
    
    // Differential Equations    
    f[0] = U*sin(u_theta) - W*cos(u_theta);         // h_dot
    f[2] = - x_Q*W - g*sin(u_theta) + qbar*S*Cx_tot/m + p_ft/m;   // U_dot
    f[3] = x_Q*U + g*cos(u_theta) + qbar*S*Cz_tot/m;  // W_dot  
    
  
} /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/



