#include "math.h"
#include "F16_AeroData_cfnc.c"              /* LOFI Look-up header file*/

void aerodata(double *xu, double *f, void *user_data){
    
//     DifferentialState x_theta x_Q x_alpha x_Vt;
//     Control u_el u_ft;
//     Parameter Czq Cz dZdQ Cz_tot Cmq Cm dMdQ Cm_tot M_tot; 
//     Parameter Cxq Cx dXdQ Cx_tot X_tot Z_tot D L;
      
  double *temp;
  double beta = 0.0;        // only longitudinal dynamics considered
  double pi   = 3.14159;
  double r2d  = 180.0/pi;   // radians to degrees 
  
    double t = xu[0];
    double x_theta = xu[1];
    double x_Q = xu[2];
    double x_alpha = xu[3]*r2d;
    double x_Vt = xu[4];       
    double u_el = xu[5];
    double u_ft = xu[6];
    double Czq = xu[7];
    double Cz = xu[8];       
    double dZdQ = xu[9];       
    double Cz_tot = xu[10];       
    double Cmq = xu[11];       
    double Cm = xu[12];       
    double dMdQ = xu[13];
    double Cm_tot = xu[14];
    double M_tot = xu[15];
    double Cxq = xu[16];
    double Cx = xu[17];
    double dXdQ = xu[18];
    double Cx_tot = xu[19];
    double X_tot = xu[20];       
    double Z_tot = xu[21];       
    double D = xu[22];       
    double L = xu[23];
  
//   double Cz, Czq;
//   double Cm, Cmq;
//   double Cy;
//   double Cyr, Cyp;
//   double Cn;
//   double Cnr, Cnp;
//   double Cl;
//   double Clr, Clp;

  temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

  /* %%%%%%%%%%%%%%%%%%
        Table lookup
     %%%%%%%%%%%%%%%%%% */    

    damping(x_alpha,temp);
        Cxq = temp[0];
        //Cyr = temp[1];
        //Cyp = temp[2];
        Czq = temp[3];
        //Clr = temp[4];
        //Clp = temp[5];
        Cmq = temp[6];
        //Cnr = temp[7];
        //Cnp = temp[8];

    cxcm(x_alpha,u_el,temp);
        Cx = temp[0];
        Cm = temp[1];

    cz(x_alpha,beta,u_el,temp);
        Cz = temp[0];
                            
free(temp); 
    
//     % constant defs
//     % f16_constants 
  	double g    = 32.17;   //% gravity, ft/s^2 
  	double m    = 636.94;  //% mass, slugs 
  	double B    = 30.0;    //% span, ft 
  	double S    = 300.0;   //% planform area, ft^2 
  	double cbar = 11.32;   //% mean aero chord, ft 
  	double xcgr = 0.35;    //% reference center of gravity as a fraction of cbar 
  	double xcg  = 0.30;    //% center of gravity as a fraction of cbar. 
  	double Heng = 0.0;     //% turbine momentum along roll axis. 

/*  altitude dependent density & dynamic pressure
    using NASA's atmospheric model */
    double x0_alt = 15000;
    double rho0 = 2.377e-3;     //% air density
    double rho;
    double tfac;
    double qbar;
    tfac =1 - .703e-5*(x0_alt);//% temp factor, using current altitude!
    rho = rho0*pow(tfac,4.14);
    qbar = .5*rho*pow(x_Vt,2.0);      //% dynamic pressure as an algebraic state?
    
//     % Inertia data
  	double Jy  = 55814.0;  //% slug-ft^2  
  	double Jxz = 982.0;    //% slug-ft^2      
  	double Jz  = 63100.0;  //% slug-ft^2 
  	double Jx  = 9496.0;   //% slug-ft^2 
    
//     %% Intermediate states
        
//     % Cz_tot
    dZdQ = (cbar/(2*x_Vt))*Czq;
    Cz_tot = Cz + dZdQ*x_Q;

//     % Cm_tot
    dMdQ = (cbar/(2*x_Vt))*Cmq;
    Cm_tot = Cm + Cz_tot*(xcgr-xcg) + dMdQ*x_Q; //% replace with x_Q ??
     
//     % total moments from coefficients, M_tot 
    M_tot = qbar*S*cbar*Cm_tot;
   
//     % Cx_tot
    dXdQ = (cbar/(2*x_Vt))*Cxq;
    Cx_tot = Cx + dXdQ*x_Q; //% replace with x_Q ??

//     % total longitudinal forces from coefficients, X_tot and Z_tot
    X_tot = qbar*S*Cx_tot;
    Z_tot = qbar*S*Cz_tot;
    
    
//     % drag D, lateral components neglected!
    D = -X_tot*cos(x_alpha)-Z_tot*sin(x_alpha);    
//     % lift L, lateral components neglected!
    L = X_tot*sin(x_alpha)-Z_tot*cos(x_alpha);
    
//     %% Differential Equation
    
    f[0] =  x_Q;    //% theta_dot
    f[1] =  M_tot/Jy;   //% q_dot
    
//     % option: include longitudinal stab. speeds in the controls if required
    f[2] = (1/m)*(u_ft*cos(x_alpha)-D-m*g*sin(x_theta-x_alpha));
    f[3] = -u_ft*sin(x_alpha)-L+m*g*cos(x_theta-x_alpha)+m*x_Vt*x_Q;   
  
} /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/



