#include "math.h"

/*  look-up tables  */

#include "F16_AeroData.c"              /* LOFI Look-up header file*/

void aerodata(double*,double*);

/*########################################*/
/*### Added for mex function in matlab ###*/
/*########################################*/

int fix(double);
int sign(double);

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{

#define XU prhs[0]
#define COEFFY plhs[0]

int i;
double *xup, *coeffp;

if (mxGetM(XU)==2 && mxGetN(XU)==1){ 

      /* Calling Program */
      xup = mxGetPr(XU);
      COEFFY = mxCreateDoubleMatrix(6, 1, mxREAL);
      coeffp = mxGetPr(COEFFY);

      aerodata(xup,coeffp);

      /* debug
      for (i=0;i<=14;i++){
        printf("coeffp(%d) = %e\n",i+1,coeffp[i]);
      }
      end debug */

} /* End if */
else{ 
      mexErrMsgTxt("Input and/or output is wrong size.");
} /* End else */

} /* end mexFunction */

/*########################################*/
/*########################################*/

void aerodata(double *xu, double *coeff){
    
    
  double pi   = acos(-1);
  double r2d;                   /* radians to degrees */
  
  double *temp;

  double alpha, beta;
  double el;

  double Cx, Cxq;
  double Cz, Czq;
  double Cm, Cmq;
  double Cy;
  double Cyr, Cyp;
  double Cn;
  double Cnr, Cnp;
  double Cl;
  double Clr, Clp;

  temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

  r2d  = 180.0/pi;     /* radians to degrees */

  /* %%%%%%%%%%%%%%%%%%%
           States
     %%%%%%%%%%%%%%%%%%% */
  
  alpha = xu[0]*r2d; /* angle of attack in degrees */
  beta  = 0.0; /* sideslip angle in degrees */

//   if (vt <= 0.01) {vt = 0.01;}

  /* %%%%%%%%%%%%%%%%%%%
     Control inputs
     %%%%%%%%%%%%%%%%%%% */
  el    = xu[1];   /* Elevator setting in degrees. */


  /* %%%%%%%%%%%%%%%%%%
        Table lookup
     %%%%%%%%%%%%%%%%%% */    

    damping(alpha,temp);
        Cxq = temp[0];
        //Cyr = temp[1];
        //Cyp = temp[2];
        Czq = temp[3];
        //Clr = temp[4];
        //Clp = temp[5];
        Cmq = temp[6];
        //Cnr = temp[7];
        //Cnp = temp[8];

    cxcm(alpha,el,temp);
        Cx = temp[0];
        Cm = temp[1];

    cz(alpha,beta,el,temp);
        Cz = temp[0];
        
        // Coefficient output
        coeff[0]=Cxq;
        coeff[1]=Czq;
        coeff[2]=Cmq;
        coeff[3]=Cx;
        coeff[4]=Cm;
        coeff[5]=Cz;
        
              
free(temp);

}; /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/


/*########################################*/
/*### Port from matlab fix() function ####*/
/*########################################*/

int fix(double in){
    int out;

    if (in >= 0.0){
       out = (int)floor(in);
    }
    else if (in < 0.0){
       out = (int)ceil(in);
    }

    return out;
}

/* port from matlab sign() function */
int sign(double in){
    int out;

    if (in > 0.0){
       out = 1;
    }
    else if (in < 0.0){
       out = -1;
    }
    else if (in == 0.0){
       out = 0;
    }
    return out;
}

/*########################################*/
/*########################################*/

