function fnc_out = altitude_autopilot(long_states, altitude_ref, t0)
%----------------------------------------------------------
% A wrapper for altitude_NMPC_RUN function
%----------------------------------------------------------

% extract states from func. input
x0_h    = long_states(1);
p0_theta= long_states(2);
p0_Vt   = long_states(3);
p0_alpha= long_states(4);
% x0_Q    = long_states(5);
uC0_gamma= p0_theta-p0_alpha;

% initialize with current state value for continuity
xu0 = x0_h; % 
p0 = [p0_alpha, p0_Vt]; % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)

T_s = 0.5;     % NB!!! sampling time not accepted as acado mex input
T_h = 5.0;      % pridiction and control horizon
N = T_h/T_s;    % 10.0;

% reference trajectory      

d2r = pi/180.0; % degrees to radians
r2d = 180.0/pi; % radians to degrees
r_h = altitude_ref;% altitude reference [ft]
r_gamma = uC0_gamma*d2r;    % flight path ref [rad]

% optimization reference
ref = [ r_h  r_gamma ];    % running cost reference
sRef= [ r_h  r_gamma ];    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)

xS1_h = x0_h+(r_h-x0_h)*0.632; % assuming tS = time constant
xS1_gamma = uC0_gamma+(r_gamma-uC0_gamma)*0.632; % assuming tS= time constant

xS2_h = x0_h+(r_h-x0_h)*0.95; %
xS2_gamma = uC0_gamma+(r_gamma-uC0_gamma)*0.95; %

% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];

h_ref0    = [x0_h xS1_h xS2_h r_h];
gamma_ref0= [uC0_gamma xS1_gamma xS2_gamma r_gamma];

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
% time_ref = t0: T_s/(t0+T_h) : (t0+T_h); 
time_ref = t0: T_h/N : (t0+T_h); 
h_ref = pchip(time_ref0, h_ref0, time_ref);
gamma_ref = pchip(time_ref0, gamma_ref0, time_ref);

% other constant refs
% gamma_ref   = r_gamma*ones(1,T_h/T_s+1); 

% the reference trajectory matrix for acado mex interface 
refT = [time_ref', h_ref', gamma_ref'];

% weighting matrices
% consider state-dependent weights: allow a relaxed control of alpha until
% alpha reaches the critical boundary regions, that is alpha<-5 or alpha>40  
Q = eye(2);           % Q and R weight matrices
Q(1,1)= 0.010;        % h, altitude
Q(1,1)= 0.1;          % gamma

% terminal weight
S = eye(2);      % P weight matrix
S(1,1)= 1000.0;  % theta
S(2,2)= 100.0;   % alpha
  
% run acado NMPC algorithm
  out = altitude_NMPC_RUN(t0, T_h, N, xu0, ref, refT, Q, p0, S, sRef,... 
        p0_alpha, p0_Vt);   % 
    
  uC_fPath = out.U(1)*r2d; % gamma ref [deg]
  p_alpha = out.P(1);
  p_Vt = out.P(2);
  
  fnc_out = [uC_fPath, p_Vt, p_alpha]';

end



