function fnc_out = fPath_autopilot(long_states, gamma_ref, alpha_ref, t0, p0_qbar, u0_ft, u0_el)
%----------------------------------------------------------
% A wrapper for pitch_NMPC function
%----------------------------------------------------------

% extract states from func. input
% x0_h    = long_states(1);
x0_theta= long_states(2); % needed to calculate gamma 
x0_Vt   = long_states(3);
x0_alpha= long_states(4);
x0_Q    = long_states(5);
x0_gamma= x0_theta-x0_alpha;
% initialize with current state value for continuity
u0_el_dot=0.0;
xu0 = [x0_gamma, x0_Q, x0_alpha, x0_Vt, u0_el, u0_el_dot]; % u0_el is the actual deflection state, not cmd!
% xu0 = [x0_theta, x0_Q, x0_alpha]; % u0_el is the actual deflection state, not cmd!
p0 = p0_qbar; % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)
% freq = 200Hz -> T_sample = 0.005, but using 0.05 instead 
% chosen T_horizon = 0.5 -> N = T_horizon/T_sample = 10

T_s = 0.05;     % NB!!! sampling time not accepted as acado mex input
T_h = 1.0;      % pridiction and control horizon
N = T_h/T_s;    % 10.0;

% reference trajectory
% options: line, spline etc to link initial and final ref values
% ref = [ startime      x0_theta  x0_Q  x0_alpha  x0_Vt  0  ft_trim
%         startime+T_h  r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];        

d2r = pi/180.0; % degrees to radians
% r2d = 180.0/pi; % radians to degrees
r_Vt = 700.0; % x0_Vt; % speed [ft/s]
r_alpha = alpha_ref*d2r;    % AoA reference [rad] 
r_gamma = gamma_ref*d2r;    % flight path ref [rad]
r_Q = 0.0;% pitch rate reference [rad/s] 
r_el = u0_el;         % el_ref;  % or use elevator trim ref setting (can be found for several operating points)
% r_elC = u0_el;        % cmd
r_ft = u0_ft;      % [lb] thrust force

% ref trajectory option 1: straight line link, t0 available in workspace
% refT = [ t0      x0_theta   r_alpha   r_el    r_Vt     r_qbar
%          t0+T_h  r_theta    r_alpha    r_el    r_Vt     r_qbar ];

% stabilizing reference trajectory for alpha: seeks stable alpha region
% control alpha into a stable region to enhance control and disturbance 
% rejection capabilities
% approach a higher alpha
% theta_err_tol = 1.0*d2r;
% theta_err = abs(r_theta - x0_theta);
% 
%  if x0_alpha < 15.0*d2r && theta_err <= theta_err_tol,
%      r_alpha = 7.5*d2r;
% %      if r_theta > 10.0*d2r,
% %          r_alpha = 16.5*d2r;    % for minimal change in control
% %      end
%      
% %  elseif x0_alpha >= 15.0*d2r && x0_alpha <= 30.0*d2r && theta_err <= theta_err_tol,
%  elseif x0_alpha >= 15.0*d2r,
%      r_alpha = 20.0*d2r;
% 
% %  elseif x0_alpha > 30.0*d2r && theta_err <= theta_err_tol
% %      r_alpha = 32.5*d2r;
%  end
       
% optimization reference
ref = [ r_gamma  r_Q  r_alpha  r_Vt  r_el  r_ft ];    % running cost reference
sRef= [ r_gamma   r_alpha ];    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)
% tS1_el= t0+(T_h/10.0);% 0.05s coincidence point= time constant for elevator

xS1_Vt = x0_Vt+(r_Vt-x0_Vt)*0.632; % assuming tS = time constant
xS1_gamma = x0_gamma+(r_gamma-x0_gamma)*0.632; % assuming tS= time constant
xS1_alpha = x0_alpha+(r_alpha-x0_alpha)*0.632; % 
% uS1_el    = u0_el+(r_el-u0_el)*0.632; % 

xS2_Vt = x0_Vt+(r_Vt-x0_Vt)*0.95; %
xS2_gamma = x0_gamma+(r_gamma-x0_gamma)*0.95; %
xS2_alpha = x0_alpha+(r_alpha-x0_alpha)*0.95; % 
% uS2_el = u0_el+(r_el-u0_el)*0.95; % 

% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];
% time_ref0_el = [t0 tS1_el tS2 t0+T_h];

Vt_ref0    = [x0_Vt xS1_Vt xS2_Vt r_Vt];
gamma_ref0= [x0_gamma xS1_gamma xS2_gamma r_gamma];
alpha_ref0= [x0_alpha xS1_alpha xS2_alpha r_alpha];  
% el_ref0   = [u0_el    uS1_el    uS2_el    r_el];  

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
% time_ref = t0: T_s/(t0+T_h) : (t0+T_h); 
time_ref = t0: T_h/N : (t0+T_h); 
Vt_ref = pchip(time_ref0, Vt_ref0, time_ref);
gamma_ref = pchip(time_ref0, gamma_ref0, time_ref);
alpha_ref = pchip(time_ref0, alpha_ref0, time_ref);
% el_ref    = pchip(time_ref0_el, el_ref0, time_ref);

% other constant refs must have same length as gamma_ref and alpha_ref
% gamma_ref   = r_gamma*ones(1,T_h/T_s+1); 
% alpha_ref   = r_alpha*ones(1,T_h/T_s+1); 
el_refC   = r_el*ones(1,T_h/T_s+1); 
ft_ref   = r_ft*ones(1,T_h/T_s+1);
Q_ref   = r_Q*ones(1,T_h/T_s+1);
% Vt_ref   = r_Vt*ones(1,T_h/T_s+1);
% qbar_ref = r_qbar*ones(1,T_h/T_s+1);

% the reference trajectory matrix for acado mex interface 
% refT = [ t0      x0_theta  x0_alpha   r_el   r_ft    r_Vt     r_qbar
%          tS      xS_theta  xS_alpha   r_el   r_ft    r_Vt     r_qbar
%          t0+T_h  r_theta   r_alpha    r_el   r_ft    r_Vt     r_qbar ];

refT = [time_ref', gamma_ref', Q_ref', alpha_ref', Vt_ref', el_refC', ft_ref'];

% weighting matrices
% consider state-dependent weights: allow a relaxed control of alpha until
% alpha reaches the critical boundary regions, that is alpha<-5 or alpha>40  
Q = eye(6);         % Q and R weight matrices
% Q(1,1)= 1.0;        % h, altitude
Q(1,1)= 100.0;    % gamma
Q(2,2)= 0.010;    % theta dot
Q(3,3)= 0.10;        % alpha
Q(4,4)= 0.0010;        % Vt
Q(5,5)= 0.001;       % uC_el,actuator deflection cmd
Q(6,6)= 1.0;       % uC_ft,thrust cmd

% let altitude rule, when alpha is within bounds.
% h_err_tol = 10.0;   % in [ft]
% h_err = abs(r_h - x0_h);
% if h_err > h_err_tol,
%     Q(2,2)= 0.0;  % gamma
%     Q(2,2)= 0.0;  % alpha    
% end

% % keep alpha within model validity bounds
% if x0_alpha < 0 || x0_alpha > 35,
%     Q(3,3)= 1.0;  % alpha
% end
% if x0_alpha < -10 || x0_alpha > 45,
%     Q(3,3)= 10.0;  % alpha
% end

% terminal weight
S = eye(2);     % P weight matrix
S(1,1)= 1000.0;  % theta
S(2,2)= 100.0;   % alpha

% input position limits
min_el = -15.0;
max_el = 15.0;  

% dynamic stabilty constraint limits
s_rho = 0.951; % e^(-T_s)=0.951
s_min_gamma = r_gamma - s_rho*abs(x0_gamma-r_gamma); 
s_max_gamma = r_gamma + s_rho*abs(x0_gamma-r_gamma); 
s_min_alpha = r_alpha - s_rho*abs(x0_alpha-r_alpha); 
s_max_alpha = r_alpha + s_rho*abs(x0_alpha-r_alpha); 
  
% run acado NMPC algorithm
  out = fPath_NMPC_RUN(t0, T_h, N, xu0, ref, refT, Q, p0, S, sRef,... 
        min_el, max_el, u0_ft, p0_qbar, s_min_gamma, s_max_gamma,... 
        s_min_alpha, s_max_alpha);   % 
    
  u_el = out.U(1);
  u_ft = out.U(2);
  p_qbar = out.P(1);
  
  fnc_out = [u_el, u_ft, p_qbar, r_alpha]';

end



