function fnc_out = fPath_autopilot_reconfig(long_states, gamma_ref, alpha_ref, t0, p0_qbar, u0_ft, u0_el1, u0_el2)
%----------------------------------------------------------
% A wrapper for fPath_NMPC_reconfig_RUN function
%----------------------------------------------------------

% extract states from func. input
% x0_h    = long_states(1);
x0_theta= long_states(2); % needed to calculate gamma 
x0_Vt   = long_states(3);
x0_alpha= long_states(4);
x0_Q    = long_states(5);
x0_gamma= x0_theta-x0_alpha;
% initialize with current state value for continuity
u0_el1_dot=0.0;
u0_el2_dot=0.0;
xu0 = [x0_gamma, x0_Q, x0_alpha, x0_Vt, u0_el1, u0_el2, u0_el1_dot, u0_el2_dot];
p0 = p0_qbar; % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)

T_s = 0.05;     % NB!!! sampling time not accepted as acado mex input
T_h = 1.0;      % pridiction and control horizon
N = T_h/T_s;    

% reference trajectory      

d2r = pi/180.0;   % degrees to radians
% r2d = 180.0/pi; % radians to degrees
r_Vt = 700.0;     % x0_Vt; % speed [ft/s]
r_alpha = alpha_ref*d2r;    % AoA reference [rad] 
r_gamma = gamma_ref*d2r;    % flight path ref [rad]
r_Q = 0.0;              % pitch rate reference [rad/s] 
r_el1 = u0_el1;         % el_ref;  
r_el2 = u0_el2;         % el_ref;  
r_ft = u0_ft;           % [lb] thrust force

% ref trajectory option 1: straight line link, t0 available in workspace

% stabilizing reference trajectory for alpha: seeks stable alpha region
% control alpha into a stable region to enhance control and disturbance 
% rejection capabilities
% approach a higher alpha
% theta_err_tol = 1.0*d2r;
% theta_err = abs(r_theta - x0_theta);
% 
%  if x0_alpha < 15.0*d2r && theta_err <= theta_err_tol,
%      r_alpha = 7.5*d2r;
% %      if r_theta > 10.0*d2r,
% %          r_alpha = 16.5*d2r;    % for minimal change in control
% %      end
%      
% %  elseif x0_alpha >= 15.0*d2r && x0_alpha <= 30.0*d2r && theta_err <= theta_err_tol,
%  elseif x0_alpha >= 15.0*d2r,
%      r_alpha = 20.0*d2r;
% 
% %  elseif x0_alpha > 30.0*d2r && theta_err <= theta_err_tol
% %      r_alpha = 32.5*d2r;
%  end
       
% optimization reference
ref = [ r_gamma  r_Q  r_alpha  r_Vt  r_el1  r_el2  r_ft ];    % running cost reference
sRef= r_gamma;    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)

xS1_Vt = x0_Vt+(r_Vt-x0_Vt)*0.632; % assuming tS = time constant
xS1_gamma = x0_gamma+(r_gamma-x0_gamma)*0.632; % assuming tS= time constant
xS1_alpha = x0_alpha+(r_alpha-x0_alpha)*0.632; % 

xS2_Vt = x0_Vt+(r_Vt-x0_Vt)*0.95; %
xS2_gamma = x0_gamma+(r_gamma-x0_gamma)*0.95; %
xS2_alpha = x0_alpha+(r_alpha-x0_alpha)*0.95; % 

% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];
% time_ref0_el = [t0 tS1_el tS2 t0+T_h];

Vt_ref0    = [x0_Vt xS1_Vt xS2_Vt r_Vt];
gamma_ref0= [x0_gamma xS1_gamma xS2_gamma r_gamma];
alpha_ref0= [x0_alpha xS1_alpha xS2_alpha r_alpha];  
% el_ref0   = [u0_el1    uS1_el    uS2_el    r_el1];  

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
time_ref = t0: T_h/N : (t0+T_h); 
Vt_ref = pchip(time_ref0, Vt_ref0, time_ref);
gamma_ref = pchip(time_ref0, gamma_ref0, time_ref);
alpha_ref = pchip(time_ref0, alpha_ref0, time_ref);

% other constant refs
el1_refC   = r_el1*ones(1,T_h/T_s+1); 
el2_refC   = r_el2*ones(1,T_h/T_s+1); 
ft_ref     = r_ft*ones(1,T_h/T_s+1);
Q_ref      = r_Q*ones(1,T_h/T_s+1);

% the reference trajectory matrix for acado mex interface 

refT = [time_ref', gamma_ref', Q_ref', alpha_ref', Vt_ref', el1_refC', el2_refC', ft_ref'];

% weighting matrices
% consider state-dependent weights: allow a relaxed control of alpha until
% alpha reaches the critical boundary regions, that is alpha<-5 or alpha>40  
Q = eye(7);         % Q and R weight matrices
Q(1,1)= 100.0;      % gamma
Q(2,2)= 0.010;      % theta dot
Q(3,3)= 0.10;       % alpha
Q(4,4)= 0.0010;     % Vt
Q(5,5)= 0.001;      % uC_el1,actuator deflection cmd
Q(6,6)= 0.001;      % uC_el2,actuator deflection cmd
Q(7,7)= 1.0;        % uC_ft,thrust cmd

% terminal weight
% S = eye(2);     % P weight matrix
S = 1000.0;  % gamma

% input position limits
min_el1 = -15.0; %-25.0
max_el1 =  15.0;  
min_el2 = -15.0;
max_el2 =  15.0;  

% input rate limits
min_el1_dot = -60.0;     % [deg/s]
max_el1_dot =  60.0;     % [deg/s]     
min_el2_dot = -60.0;     % [deg/s]
max_el2_dot =  60.0;     % [deg/s]     

% Controller Reconfiguration
% if min_el1 == max_el1 || min_el2 == max_el2,
%     
%     % make secondary state objectives less important 
%     Q(2,2)= 0.1*Q(2,2);    % theta dot
%     Q(3,3)= 0.1*Q(3,3);    % alpha
%     Q(4,4)= 0.1*Q(4,4);    % Vt
%    
%     % make redundant control surface work harder 
%     if min_el1 == max_el1 && min_el2 < max_el2,
%     Q(6,6)= 0.1*Q(6,6); % uC_el2 
%     elseif min_el2 == max_el2 && min_el1 < max_el1,
%     Q(5,5)= 0.1*Q(5,5); % uC_el1
%     end
%     
%     % NB! increase horizon, if computational power is not a constraint    
% end

% dynamic stabilty constraint limits
s_rho = 0.951; % e^(-T_s)=0.951
s_min_gamma = r_gamma - s_rho*abs(x0_gamma-r_gamma); 
s_max_gamma = r_gamma + s_rho*abs(x0_gamma-r_gamma); 
s_min_alpha = r_alpha - s_rho*abs(x0_alpha-r_alpha); 
s_max_alpha = r_alpha + s_rho*abs(x0_alpha-r_alpha); 
  
% run acado NMPC algorithm
  out = fPath_NMPC_reconfig_RUN(t0, T_h, N, xu0, ref, refT, Q, p0, S, sRef,... 
        min_el1, max_el1, min_el2, max_el2, min_el1_dot, max_el1_dot, ...
        min_el2_dot, max_el2_dot, u0_ft, p0_qbar,... 
        s_min_gamma, s_max_gamma, s_min_alpha, s_max_alpha);   % 
    
  u_el1 = out.U(1);
  u_el2 = out.U(2);
  u_ft = out.U(3);
  p_qbar = out.P(1);
  
  fnc_out = [u_el1, u_el2, u_ft, p_qbar, r_alpha]';

end



