/*
*    This file is part of ACADO Toolkit.
*
*    ACADO Toolkit -- A Toolkit for Automatic Control and Dynamic Optimization.
*    Copyright (C) 2008-2009 by Boris Houska and Hans Joachim Ferreau, K.U.Leuven.
*    Developed within the Optimization in Engineering Center (OPTEC) under
*    supervision of Moritz Diehl. All rights reserved.
*
*    ACADO Toolkit is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Lesser General Public
*    License as published by the Free Software Foundation; either
*    version 3 of the License, or (at your option) any later version.
*
*    ACADO Toolkit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Lesser General Public License for more details.
*
*    You should have received a copy of the GNU Lesser General Public
*    License along with ACADO Toolkit; if not, write to the Free Software
*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*
*/


/**
*    Author David Ariens
*    Date 2009-2010
*    http://www.acadotoolkit.org/matlab 
*/

#include <acado_optimal_control.hpp>
#include <acado_toolkit.hpp>
#include <acado/utils/matlab_acado_utils.hpp>

USING_NAMESPACE_ACADO

#include "aerodata_cfncPath.c"

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ) 
 { 
 
    clearAllStaticCounters( ); 
 
    acadoPrintf("\nACADO Toolkit for Matlab - Developed by David Ariens, 2009-2010 \n"); 
    acadoPrintf("Support available at http://www.acadotoolkit.org/matlab \n \n"); 

    if (nrhs != 18){ 
      mexErrMsgTxt("This problem expects 18 right hand side argument(s) since you have defined 18 MexInput(s)");
    } 
 
    TIME autotime;
    DifferentialState x_gamma;
    DifferentialState x_Q;
    DifferentialState x_alpha;
    DifferentialState x_Vt;
    DifferentialState u_el;
    DifferentialState u_el_dot;
    Control uC_el;
    Control uC_ft;
    Parameter p_qbar; 
    double acadoconstant1 = 0;
    Function acadodata_f2;
    acadodata_f2 << x_gamma;
    acadodata_f2 << x_Q;
    acadodata_f2 << x_alpha;
    acadodata_f2 << x_Vt;
    acadodata_f2 << uC_el;
    acadodata_f2 << uC_ft;
    double acadoconstant2 = -60;
    double acadoconstant3 = 60;
    double acadoconstant4 = -1.745329E-001;
    double acadoconstant5 = 7.853982E-001;
    double acadoconstant6 = 300;
    double acadoconstant7 = 900;
    double acadoconstant8 = -7.853982E-001;
    double acadoconstant9 = 7.853982E-001;
    double acadoconstant10 = 1000;
    double acadoconstant11 = 19000;
    double acadoconstant12 = 5.000000E-002;
    double *mexinput0_temp = NULL; 
    if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) || !(mxGetM(prhs[0])==1 && mxGetN(prhs[0])==1) ) { 
      mexErrMsgTxt("Input 0 must be a noncomplex scalar double.");
    } 
    mexinput0_temp = mxGetPr(prhs[0]); 
    double mexinput0 = *mexinput0_temp; 

    double *mexinput1_temp = NULL; 
    if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1])==1 && mxGetN(prhs[1])==1) ) { 
      mexErrMsgTxt("Input 1 must be a noncomplex scalar double.");
    } 
    mexinput1_temp = mxGetPr(prhs[1]); 
    double mexinput1 = *mexinput1_temp; 

    double *mexinput2_temp = NULL; 
    if( !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) || !(mxGetM(prhs[2])==1 && mxGetN(prhs[2])==1) ) { 
      mexErrMsgTxt("Input 2 must be a noncomplex scalar double.");
    } 
    mexinput2_temp = mxGetPr(prhs[2]); 
    double mexinput2 = *mexinput2_temp; 

    int mexinput3_count = 0;
    if (mxGetM(prhs[3]) == 1 && mxGetN(prhs[3]) >= 1) 
       mexinput3_count = mxGetN(prhs[3]);
    else if (mxGetM(prhs[3]) >= 1 && mxGetN(prhs[3]) == 1) 
       mexinput3_count = mxGetM(prhs[3]);
    else 
       mexErrMsgTxt("Input 3 must be a noncomplex double vector of dimension 1xY.");

    double *mexinput3_temp = NULL; 
    if( !mxIsDouble(prhs[3]) || mxIsComplex(prhs[3])) { 
      mexErrMsgTxt("Input 3 must be a noncomplex double vector of dimension 1xY.");
    } 
    mexinput3_temp = mxGetPr(prhs[3]); 
    Vector mexinput3(mexinput3_count);
    for( int i=0; i<mexinput3_count; ++i ){ 
        mexinput3(i) = mexinput3_temp[i];
    } 

    int mexinput4_count = 0;
    if (mxGetM(prhs[4]) == 1 && mxGetN(prhs[4]) >= 1) 
       mexinput4_count = mxGetN(prhs[4]);
    else if (mxGetM(prhs[4]) >= 1 && mxGetN(prhs[4]) == 1) 
       mexinput4_count = mxGetM(prhs[4]);
    else 
       mexErrMsgTxt("Input 4 must be a noncomplex double vector of dimension 1xY.");

    double *mexinput4_temp = NULL; 
    if( !mxIsDouble(prhs[4]) || mxIsComplex(prhs[4])) { 
      mexErrMsgTxt("Input 4 must be a noncomplex double vector of dimension 1xY.");
    } 
    mexinput4_temp = mxGetPr(prhs[4]); 
    Vector mexinput4(mexinput4_count);
    for( int i=0; i<mexinput4_count; ++i ){ 
        mexinput4(i) = mexinput4_temp[i];
    } 

    double *mexinput5_temp = NULL; 
    if( !mxIsDouble(prhs[5]) || mxIsComplex(prhs[5]) ) { 
      mexErrMsgTxt("Input 5 must be a noncomplex double vector of dimension XxY.");
    } 
    mexinput5_temp = mxGetPr(prhs[5]); 
    Matrix mexinput5_m(mxGetM(prhs[5]), mxGetN(prhs[5]));
    for( int i=0; i<mxGetN(prhs[5]); ++i ){ 
        for( int j=0; j<mxGetM(prhs[5]); ++j ){ 
           mexinput5_m(j,i) = mexinput5_temp[i*mxGetM(prhs[5]) + j];
        } 
    } 
    VariablesGrid mexinput5(mexinput5_m); 

    double *mexinput6_temp = NULL; 
    if( !mxIsDouble(prhs[6]) || mxIsComplex(prhs[6]) ) { 
      mexErrMsgTxt("Input 6 must be a noncomplex double vector of dimension XxY.");
    } 
    mexinput6_temp = mxGetPr(prhs[6]); 
    Matrix mexinput6_m(mxGetM(prhs[6]), mxGetN(prhs[6]));
    for( int i=0; i<mxGetN(prhs[6]); ++i ){ 
        for( int j=0; j<mxGetM(prhs[6]); ++j ){ 
           mexinput6_m(j,i) = mexinput6_temp[i*mxGetM(prhs[6]) + j];
        } 
    } 
    VariablesGrid mexinput6(mexinput6_m); 

    int mexinput7_count = 0;
    if (mxGetM(prhs[7]) == 1 && mxGetN(prhs[7]) >= 1) 
       mexinput7_count = mxGetN(prhs[7]);
    else if (mxGetM(prhs[7]) >= 1 && mxGetN(prhs[7]) == 1) 
       mexinput7_count = mxGetM(prhs[7]);
    else 
       mexErrMsgTxt("Input 7 must be a noncomplex double vector of dimension 1xY.");

    double *mexinput7_temp = NULL; 
    if( !mxIsDouble(prhs[7]) || mxIsComplex(prhs[7])) { 
      mexErrMsgTxt("Input 7 must be a noncomplex double vector of dimension 1xY.");
    } 
    mexinput7_temp = mxGetPr(prhs[7]); 
    Vector mexinput7(mexinput7_count);
    for( int i=0; i<mexinput7_count; ++i ){ 
        mexinput7(i) = mexinput7_temp[i];
    } 

    double *mexinput8_temp = NULL; 
    if( !mxIsDouble(prhs[8]) || mxIsComplex(prhs[8]) ) { 
      mexErrMsgTxt("Input 8 must be a noncomplex double vector of dimension XxY.");
    } 
    mexinput8_temp = mxGetPr(prhs[8]); 
    Matrix mexinput8_m(mxGetM(prhs[8]), mxGetN(prhs[8]));
    for( int i=0; i<mxGetN(prhs[8]); ++i ){ 
        for( int j=0; j<mxGetM(prhs[8]); ++j ){ 
           mexinput8_m(j,i) = mexinput8_temp[i*mxGetM(prhs[8]) + j];
        } 
    } 
    VariablesGrid mexinput8(mexinput8_m); 

    int mexinput9_count = 0;
    if (mxGetM(prhs[9]) == 1 && mxGetN(prhs[9]) >= 1) 
       mexinput9_count = mxGetN(prhs[9]);
    else if (mxGetM(prhs[9]) >= 1 && mxGetN(prhs[9]) == 1) 
       mexinput9_count = mxGetM(prhs[9]);
    else 
       mexErrMsgTxt("Input 9 must be a noncomplex double vector of dimension 1xY.");

    double *mexinput9_temp = NULL; 
    if( !mxIsDouble(prhs[9]) || mxIsComplex(prhs[9])) { 
      mexErrMsgTxt("Input 9 must be a noncomplex double vector of dimension 1xY.");
    } 
    mexinput9_temp = mxGetPr(prhs[9]); 
    Vector mexinput9(mexinput9_count);
    for( int i=0; i<mexinput9_count; ++i ){ 
        mexinput9(i) = mexinput9_temp[i];
    } 

    double *mexinput10_temp = NULL; 
    if( !mxIsDouble(prhs[10]) || mxIsComplex(prhs[10]) || !(mxGetM(prhs[10])==1 && mxGetN(prhs[10])==1) ) { 
      mexErrMsgTxt("Input 10 must be a noncomplex scalar double.");
    } 
    mexinput10_temp = mxGetPr(prhs[10]); 
    double mexinput10 = *mexinput10_temp; 

    double *mexinput11_temp = NULL; 
    if( !mxIsDouble(prhs[11]) || mxIsComplex(prhs[11]) || !(mxGetM(prhs[11])==1 && mxGetN(prhs[11])==1) ) { 
      mexErrMsgTxt("Input 11 must be a noncomplex scalar double.");
    } 
    mexinput11_temp = mxGetPr(prhs[11]); 
    double mexinput11 = *mexinput11_temp; 

    double *mexinput12_temp = NULL; 
    if( !mxIsDouble(prhs[12]) || mxIsComplex(prhs[12]) || !(mxGetM(prhs[12])==1 && mxGetN(prhs[12])==1) ) { 
      mexErrMsgTxt("Input 12 must be a noncomplex scalar double.");
    } 
    mexinput12_temp = mxGetPr(prhs[12]); 
    double mexinput12 = *mexinput12_temp; 

    double *mexinput13_temp = NULL; 
    if( !mxIsDouble(prhs[13]) || mxIsComplex(prhs[13]) || !(mxGetM(prhs[13])==1 && mxGetN(prhs[13])==1) ) { 
      mexErrMsgTxt("Input 13 must be a noncomplex scalar double.");
    } 
    mexinput13_temp = mxGetPr(prhs[13]); 
    double mexinput13 = *mexinput13_temp; 

    double *mexinput14_temp = NULL; 
    if( !mxIsDouble(prhs[14]) || mxIsComplex(prhs[14]) || !(mxGetM(prhs[14])==1 && mxGetN(prhs[14])==1) ) { 
      mexErrMsgTxt("Input 14 must be a noncomplex scalar double.");
    } 
    mexinput14_temp = mxGetPr(prhs[14]); 
    double mexinput14 = *mexinput14_temp; 

    double *mexinput15_temp = NULL; 
    if( !mxIsDouble(prhs[15]) || mxIsComplex(prhs[15]) || !(mxGetM(prhs[15])==1 && mxGetN(prhs[15])==1) ) { 
      mexErrMsgTxt("Input 15 must be a noncomplex scalar double.");
    } 
    mexinput15_temp = mxGetPr(prhs[15]); 
    double mexinput15 = *mexinput15_temp; 

    double *mexinput16_temp = NULL; 
    if( !mxIsDouble(prhs[16]) || mxIsComplex(prhs[16]) || !(mxGetM(prhs[16])==1 && mxGetN(prhs[16])==1) ) { 
      mexErrMsgTxt("Input 16 must be a noncomplex scalar double.");
    } 
    mexinput16_temp = mxGetPr(prhs[16]); 
    double mexinput16 = *mexinput16_temp; 

    double *mexinput17_temp = NULL; 
    if( !mxIsDouble(prhs[17]) || mxIsComplex(prhs[17]) || !(mxGetM(prhs[17])==1 && mxGetN(prhs[17])==1) ) { 
      mexErrMsgTxt("Input 17 must be a noncomplex scalar double.");
    } 
    mexinput17_temp = mxGetPr(prhs[17]); 
    double mexinput17 = *mexinput17_temp; 

    DifferentialEquation acadodata_f1;
    IntermediateState setc_is_1(10);
    setc_is_1(0) = autotime;
    setc_is_1(1) = x_gamma;
    setc_is_1(2) = x_Q;
    setc_is_1(3) = x_alpha;
    setc_is_1(4) = x_Vt;
    setc_is_1(5) = u_el;
    setc_is_1(6) = u_el_dot;
    setc_is_1(7) = uC_el;
    setc_is_1(8) = uC_ft;
    setc_is_1(9) = p_qbar;
    CFunction cLinkModel_1( 6, aerodata ); 
    acadodata_f1 << cLinkModel_1(setc_is_1); 

    OCP ocp1(acadoconstant1, mexinput1, mexinput2);
    ocp1.minimizeLSQ(mexinput6_m, acadodata_f2, mexinput4);
    ocp1.subjectTo(acadodata_f1);
    ocp1.subjectTo(mexinput10 <= uC_el <= mexinput11);
    ocp1.subjectTo(acadoconstant2 <= u_el_dot <= acadoconstant3);
    ocp1.subjectTo(acadoconstant4 <= x_alpha <= acadoconstant5);
    ocp1.subjectTo(acadoconstant6 <= x_Vt <= acadoconstant7);
    ocp1.subjectTo(acadoconstant8 <= x_gamma <= acadoconstant9);
    ocp1.subjectTo(acadoconstant10 <= uC_ft <= acadoconstant11);
    ocp1.subjectTo(mexinput13 == p_qbar);

    RealTimeAlgorithm algo1(ocp1, acadoconstant12);

    StaticReferenceTrajectory referencetrajectory(mexinput5);
    Controller controller1( algo1,referencetrajectory );
    controller1.init(mexinput0, mexinput3, mexinput7);
    controller1.step(mexinput0, mexinput3);

    const char* outputFieldNames[] = {"U", "P"}; 
    plhs[0] = mxCreateStructMatrix( 1,1,2,outputFieldNames ); 
    mxArray *OutU = NULL;
    double  *outU = NULL;
    OutU = mxCreateDoubleMatrix( 1,controller1.getNU(),mxREAL ); 
    outU = mxGetPr( OutU );
    Vector vec_outU; 
    controller1.getU(vec_outU); 
    for( int i=0; i<vec_outU.getDim(); ++i ){ 
        outU[i] = vec_outU(i); 
    } 

    mxArray *OutP = NULL;
    double  *outP = NULL;
    OutP = mxCreateDoubleMatrix( 1,controller1.getNP(),mxREAL ); 
    outP = mxGetPr( OutP );
    Vector vec_outP; 
    controller1.getP(vec_outP); 
    for( int i=0; i<vec_outP.getDim(); ++i ){ 
        outP[i] = vec_outP(i); 
    } 

    mxSetField( plhs[0],0,"U",OutU );
    mxSetField( plhs[0],0,"P",OutP );


    clearAllStaticCounters( ); 
 
} 

