function fnc_out = heading_autopilot(lat_states, psi_ref, t0)
%----------------------------------------------------------
% A wrapper for heading_NMPC_RUN function
%----------------------------------------------------------

% extract states and parameters from func. input
uC0_phi    = lat_states(1);
% p0_theta  = lat_states(2);
x0_psi = lat_states(3); 
p0_Vt      = lat_states(4);
% p0_alpha  = lat_states(5);
d0_beta   = lat_states(6); % disturbance
% x0_P      = lat_states(7);
% x0_R      = lat_states(8);

% initialize with current state value for continuity
xu0 = x0_psi; % 
p0  = p0_Vt;  % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)

T_s = 0.5;     % NB!!! sampling time not accepted as acado mex input
T_h = 5.0;      % 2s tested, with good results! prediction and control horizon
N = T_h/T_s;    % 

% reference trajectory       

d2r = pi/180.0; % degrees to radians
r2d = 180.0/pi; % radians to degrees
r_psi = psi_ref*d2r;% heading reference [rad]
r_phi = uC0_phi; % roll reference [rad] 
       
% optimization reference
ref = [ r_psi  r_phi ];    % running cost reference
sRef= r_psi;    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)

xS1_psi = x0_psi+(r_psi-x0_psi)*0.632; % 0.25, 0.632 assuming tS = time constant
xS2_psi = x0_psi+(r_psi-x0_psi)*0.95;  % 0.75, 0.95 


% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];

psi_ref0    = [x0_psi xS1_psi xS2_psi r_psi]; 

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
time_ref = t0: T_h/N : (t0+T_h); 
psi_ref  = pchip(time_ref0, psi_ref0, time_ref);
% phi_ref  = pchip(time_ref0, phi_ref0, time_ref);

% other constant refs
phi_ref   = r_phi*ones(1,T_h/T_s+1); 

refT = [time_ref', psi_ref', phi_ref'];

% weighting matrices 
Q_R = eye(2);          % Q and R weight matrices
Q_R(1,1)= 2.50;        % 2.5 (good for a1 fault) 1.10, 10.10 psi, heading
Q_R(2,2)= 0.10;        % 0.10, phi, roll

% terminal weight
S = eye(2);     % P weight matrix
S(1,1)= 1000.0;  % theta
S(2,2)= 100.0;   % alpha
  
% run acado NMPC algorithm
la_out = heading_NMPC_RUN(t0, T_h, N, xu0, ref, refT, Q_R, p0, S, sRef,... 
                            p0_Vt, d0_beta);
    
  uC_roll = la_out.U(1)*r2d; % roll cmd in [deg]
  p_Vt = la_out.P(1);
  
  fnc_out = [uC_roll, p_Vt]';
%   fnc_out = uC_roll;

end



