% function u_el = pitch_autopilot(long_states, pitch_ref, qbar, Mt0, Xt0, Zt0, t0)
%
% A wrapper for pitch_NMPC function
%

% clear;

% extract states from func. input
% x0_alt  = 1.0; % long_states(1);
x0_theta= 1.0; % long_states(2);
x0_Vt   = 1.0; % long_states(3);
x0_alpha= 1.0; % long_states(4);
x0_Q    = 1.0; % long_states(5);
xu0 = [x0_theta, x0_Q];
p0 = [x0_alpha, x0_Vt];

t0=0.0; %input

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)
% freq = 200Hz -> T_sample = 0.005 
% chosen T_horizon = 0.05 -> N = T_horizon/T_sample = 10

% T_s = 0.005;    % sampling time not accepted as acado mex input
T_h = 1.0;     % pridiction and control horizon
N = 20.0;

% reference trajectory
% options: line, spline etc to link initial and final ref values
% ref = [ startime      x0_theta  x0_Q  x0_alpha  x0_Vt  0  ft_trim
%         startime+T_h  r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];        

r_theta = 15.0; % $just for test$ = pitch_ref; % pitch reference [deg]  

r_Q     = 20.0;     % pitch rate ref [deg/s]
r_alpha = 20.0;     % trim angle of attack   
r_Vt    = 350.0;    % trim velocity   
r_el    = 0.0;      % ref elevator deflection
ft_trim = 350.0;    % trim thrust 

% ref trajectory option 1: straight line link, t0 available in workspace
% refT = [ t0      x0_theta  x0_Q  x0_alpha  x0_Vt  r_el  ft_trim
%          t0+T_h  r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
     
refT = [ t0      x0_theta 
         t0+T_h  r_theta ];
      
% optimization reference
% ref = [ r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
ref = [ r_theta ];
      
% weighting matrix
Q = 1000.0;  
                     

% EXECUTE MULTIPLE STEPS
u_el = [];
j=2;

% trim states
states = ones(12,1); % 12
states(3) = 20000.0;
states(7) = 350.0;
states(8) = 17.0;

% controls = ones(4,1); % 4
controls = [4193.77859144268, -4.60885629108611, 3.03206454456930e-15, -1.21163052584819e-14]'; %  trim controls
u_el(1) = controls(2);

% y_out = zeros(25,1);

for i=0.0 : 1.0 : 2.0
      
    contrl_out = pitch_NMPC_RUN(0, T_h, N, xu0, ref, refT, Q, p0);   % 

    controls(2) = contrl_out.U(1);
    u_el(j)=controls(2);
    j=j+1;
    
    fnc_input = [states; controls; 0; 0; 0];
%     y_out = nlplant(fnc_input);
    y_out = feval('nlplant',fnc_input);
    
    states(1:12) = trapz(y_out(1:12));    
    
end

% end



