#include "math.h"

/*  look-up tables  */

#include "lofi_F16_AeroData.c"              /* LOFI Look-up header file*/

void aerodata(double*,double*);

/*########################################*/
/*### Added for mex function in matlab ###*/
/*########################################*/

int fix(double);
int sign(double);

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{

#define XU prhs[0]
#define COEFFY plhs[0]

int i;
double *xup, *coeffp;

if (mxGetM(XU)==2 && mxGetN(XU)==1){ 

      /* Calling Program */
      xup = mxGetPr(XU);
      COEFFY = mxCreateDoubleMatrix(6, 1, mxREAL);
      coeffp = mxGetPr(COEFFY);

      aerodata(xup,coeffp);

      /* debug
      for (i=0;i<=14;i++){
        printf("coeffp(%d) = %e\n",i+1,coeffp[i]);
      }
      end debug */

} /* End if */
else{ 
      mexErrMsgTxt("Input and/or output is wrong size.");
} /* End else */

} /* end mexFunction */

/*########################################*/
/*########################################*/

void aerodata(double *xu, double *coeff){

  double *temp;

  double npos, epos, alt, phi, theta, psi, vt, alpha, beta, P, Q, R;
  double T, el, ail, rud, dail, drud, lef, dlef;

  double Cx_tot, Cx, delta_Cx_lef, dXdQ, Cxq, delta_Cxq_lef;
  double Cz_tot, Cz, delta_Cz_lef, dZdQ, Czq, delta_Czq_lef;
  double Cm_tot, Cm, eta_el, delta_Cm_lef, dMdQ, Cmq, delta_Cmq_lef, delta_Cm, delta_Cm_ds;
  double Cy_tot, Cy, delta_Cy_lef, dYdail, delta_Cy_r30, dYdR, dYdP;
  double delta_Cy_a20, delta_Cy_a20_lef, Cyr, delta_Cyr_lef, Cyp, delta_Cyp_lef;
  double Cn_tot, Cn, delta_Cn_lef, dNdail, delta_Cn_r30, dNdR, dNdP, delta_Cnbeta;
  double delta_Cn_a20, delta_Cn_a20_lef, Cnr, delta_Cnr_lef, Cnp, delta_Cnp_lef;
  double Cl_tot, Cl, delta_Cl_lef, dLdail, delta_Cl_r30, dLdR, dLdP, delta_Clbeta;
  double delta_Cl_a20, delta_Cl_a20_lef, Clr, delta_Clr_lef, Clp, delta_Clp_lef;

  temp = (double *)malloc(9*sizeof(double));  /*size of 9.1 array*/

  r2d  = 180.0/pi;     /* radians to degrees */

  /* %%%%%%%%%%%%%%%%%%%
           States
     %%%%%%%%%%%%%%%%%%% */

//   npos  = xu[0];   /* north position */
//   epos  = xu[1];   /* east position */
//   alt   = xu[2];   /* altitude */
//   phi   = xu[3];   /* orientation angles in rad. */
//   theta = xu[4];
//   psi   = xu[5];

//   vt    = xu[6];     /* total velocity */
  alpha = xu[0]*r2d; /* angle of attack in degrees */
  beta  = 0.0; /* sideslip angle in degrees */
//   P     = xu[9];     /* Roll Rate --- rolling  moment is Lbar */
//   Q     = xu[10];    /* Pitch Rate--- pitching moment is M */
//   R     = xu[11];    /* Yaw Rate  --- yawing   moment is N */

//   if (vt <= 0.01) {vt = 0.01;}

  /* %%%%%%%%%%%%%%%%%%%
     Control inputs
     %%%%%%%%%%%%%%%%%%% */

//   T     = xu[12];   /* thrust */
  el    = xu[1];   /* Elevator setting in degrees. */
//   ail   = xu[14];   /* Ailerons mex setting in degrees. */
//   rud   = xu[15];   /* Rudder setting in degrees. */
//   lef   = xu[16];   /* Leading edge flap setting in degrees */
    
  /* dail  = ail/20.0;   aileron normalized against max angle */
  /* The aileron was normalized using 20.0 but the NASA report and
     S&L both have 21.5 deg. as maximum deflection. */
  /* As a result... */
//   dail  = ail/21.5;
//   drud  = rud/30.0;  /* rudder normalized against max angle */
//   dlef  = (1 - lef/25.0);  /* leading edge flap normalized against max angle */


/* %%%%%%%%%%%%%%%%%%
        Table lookup
     %%%%%%%%%%%%%%%%%% */

/* The lofi model does not include the
   leading edge flap.  All terms multiplied
   dlef have been set to zero but just to 
   be sure we will set it to zero. */
    
//     dlef = 0.0;     

    damping(alpha,temp);
        Cxq = temp[0];
        //Cyr = temp[1];
        //Cyp = temp[2];
        Czq = temp[3];
        //Clr = temp[4];
        //Clp = temp[5];
        Cmq = temp[6];
        //Cnr = temp[7];
        //Cnp = temp[8];

//     clcn(alpha,beta,temp);
//         Cl = temp[0];
//         Cn = temp[1];

    cxcm(alpha,el,temp);
        Cx = temp[0];
        Cm = temp[1];

//     Cy = -.02*beta + .021*dail + .086*drud;

    cz(alpha,beta,el,temp);
        Cz = temp[0];
        
        // Coefficient output
        coeff[0]=Cxq;
        coeff[1]=Czq;
        coeff[2]=Cmq;
        coeff[3]=Cx;
        coeff[4]=Cm;
        coeff[5]=Cz;
        
              
free(temp);

}; /*##### END of aerodata() ####*/



/*########################################*/
/*########################################*/


/*########################################*/
/*### Port from matlab fix() function ####*/
/*########################################*/

int fix(double in){
    int out;

    if (in >= 0.0){
       out = (int)floor(in);
    }
    else if (in < 0.0){
       out = (int)ceil(in);
    }

    return out;
}

/* port from matlab sign() function */
int sign(double in){
    int out;

    if (in > 0.0){
       out = 1;
    }
    else if (in < 0.0){
       out = -1;
    }
    else if (in == 0.0){
       out = 0;
    }
    return out;
}

/*########################################*/
/*########################################*/

