%--------------------------------------------------------------------------
% file/function name: pitch_NMPC 
%
% This file (pitch_NMPC.m) is compiled by using the matlab run button (F5). 
% After compilation the file pitch_NMPC_RUN can be run multiple times with 
% different parameters without needing to recompile again.
%  
% REMARK: the sequence of the parameters must be the same sequence as
%         how they were defined:
%
% run: 
% pitch_NMPC_RUN(x0_alt, x0_theta, x0_Vt, x0_alpha, x0_Q, u0_el, u0_ft,
%                xu0{v}, t0, Q{m}, xRef{v}, xRefT{m}, Mt0, Xt0, Zt0, qbar0)
%
%--------------------------------------------------------------------------

clear;

BEGIN_ACADO; 
    
    % Acado function name property
    acadoSet('problemname', 'pitch_NMPC');
       
    
    DifferentialState x_theta;  % define longitudinal states for pitch control
    DifferentialState x_Q;
    DifferentialState x_alpha;
    DifferentialState x_Vt;
         
    Control u_el;   % elevator deflection % define longitudinal controls for pitch
    Control u_ft;   % thrust force
            
%     x0_alt =    acado.MexInput; % read states and other config. matrices as mex input
%     x0_theta =  acado.MexInput; % use mexinput for each state: mex vector input does not split?      
%     x0_Vt =     acado.MexInput;
%     x0_alpha =  acado.MexInput;
%     x0_Q =      acado.MexInput;

%     qbar0 = acado.MexInput;     % dynamic pressure, constant for each call?    
%     t0 =    acado.MexInput;        % current sim time    
%     T_h =   acado.MexInput;       % T_horizon
%     N =     acado.MexInput;         % number of intervals in horizon
%     xu0 =   acado.MexInputVector; % vector input necessary for sim init 
%     xRef =  acado.MexInputVector;
%     xRefT = acado.MexInputMatrix; 
%     Q =     acado.MexInputMatrix;   % matching the cost state variables


    x0_alpha =  acado.MexInput;
    u0_el =  acado.MexInput;
    qbar0 = acado.MexInput;     % dynamic pressure, constant for each call?
    t0 =    acado.MexInput;        % current sim time    
    T_h =   acado.MexInput;       % T_horizon
    N =     acado.MexInput;         % number of intervals in horizon
    xu0 =   acado.MexInputVector; % vector input necessary for sim init 
    xRef =  acado.MexInputVector;
    xRefT = acado.MexInputMatrix; 
    Q =     acado.MexInputMatrix;   % matching the cost state variables

    %******************* test start ******************************
    
    
    % extract states from func. input
x0_alt  = 1.0; % long_states(1);
x0_theta= 1.0; % long_states(2);
x0_Vt   = 1.0; % long_states(3);
% x0_alpha= 1.0; % long_states(4);
x0_Q    = 1.0; % long_states(5);
% xu0 = [x0_theta, x0_Vt, x0_alpha, x0_Q, 0.0, 5000.0];

% T_h = 0.05;     % pridiction and control horizon
% N = 10.0;       

% r_theta = 15.0; % $just for test$ = pitch_ref; % pitch reference [deg]  
% 
% r_Q     = 20.0;     % pitch rate ref [deg/s]
% r_alpha = 20.0;     % trim angle of attack   
% r_Vt    = 350.0;    % trim velocity   
% r_el    = 0.0;      % ref elevator deflection
% ft_trim = 350.0;    % trim thrust 

% t0=0.0;

% ref trajectory option 1: straight line link, t0 available in workspace
% xRefT = [ t0      x0_theta  x0_Q  x0_alpha  x0_Vt  r_el  ft_trim
%          t0+T_h  r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
      
% optimization reference
% xRef = [ r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
      
% weighting matrix
% Q = eye(6);
% Q(1,1)= 100.0;


% just for test
Mt0 = 1.0; %
Xt0 = 1.0; %
Zt0 = 1.0; %
% qbar = 0.5;   
    
    %******************* test end ******************************
    
    
    
%     u0_el =     0.0;    % elevator deflection [deg]
    u0_ft =     5000.0; % thrust [lb]    
    
    % current sim time for start of a new prediction horizon
    startime =  t0;    
    
    T_s = 0.005;     % T_sample, acado doesn't accept mex input!
    t_start	= 0.0;  % start of horizon ????
    t_end	= T_h;  % end/length of horizon
    
    % constant defs
    % f16_constants 
  	g    = 32.17;   % gravity, ft/s^2 
  	m    = 636.94;  % mass, slugs 
  	B    = 30.0;    % span, ft 
  	S    = 300.0;   % planform area, ft^2 
  	cbar = 11.32;   % mean aero chord, ft 
  	xcgr = 0.35;    % reference center of gravity as a fraction of cbar 
  	xcg  = 0.30;    % center of gravity as a fraction of cbar. 
  	Heng = 0.0;     % turbine momentum along roll axis. 
    
  	r2d  = 180.0/pi;% radians to degrees 

%     % altitude dependent density & dynamic pressure
%     rho0 = 2.377e-3;  % air density
%     tfac =1 - .703e-5*(x0_alt); % temp factor, using current altitude!
%     rho = rho0*tfac^4.14;
%     qbar = .5*rho*x_Vt^2;   % dynamic pressure as an algebraic state?
    
    % Inertia data
  	Jy  = 55814.0;  % slug-ft^2  
  	Jxz = 982.0;    % slug-ft^2      
  	Jz  = 63100.0;  % slug-ft^2 
  	Jx  = 9496.0;   % slug-ft^2 
    
    % f16 operating range
    min_el = -15.0;
    max_el = 15.0;
    min_Vt = 300;
    max_Vt = 900;
    min_ft = 1000.0;
    max_ft = 19000.0;
    min_alpha = -10.0;
    max_alpha = 45.0;
       

    %% Intermediate states

    % find moment coefficients (once for each call or calculate stepwise?)
    % if once for each call, define M_tot as a mex input!

    % call lookup function for coefficients
    
%     aeroparams = [x0_alpha; u0_el];
    aeroparams = [x_alpha; u_el];
    
    Coeffs = aerodata_long(aeroparams);
    
    % Cz_tot
    Czq = Coeffs(2);    % from lookup table
    Cz  = Coeffs(6);     % from lookup table
    dZdQ = (cbar/(2*x_Vt))*Czq;
    Cz_tot = Cz + dZdQ*x_Q;

    % Cm_tot
    Cmq = Coeffs(3);    % from lookup table
    Cm  = Coeffs(5);     % from lookup table
    dMdQ = (cbar/(2*x_Vt))*Cmq;
    Cm_tot = Cm + Cz_tot*(xcgr-xcg) + dMdQ*x_Q; % replace with x_Q ??
    
    % qbar = .5*rho*x_Vt^2;
    
    % total moments from coefficients, M_tot 
    M_tot = qbar0*S*cbar*Cm_tot;
    
    % find force coefficients
    % if once for each call, define X_tot and Z_tot as mex inputs!
   
    % Cx_tot
    Cxq = Coeffs(1);    % from lookup table
    Cx = Coeffs(4);     % from lookup table
    dXdQ = (cbar/(2*x_Vt))*Cxq;
    Cx_tot = Cx + dXdQ*x_Q; % replace with x_Q ??

    % total longitudinal forces from coefficients, X_tot and Z_tot
    X_tot = qbar0*S*Cx_tot;
    Z_tot = qbar0*S*Cz_tot;
    
    
    % drag D, lateral components neglected!
    D = -X_tot*cos(x_alpha)-Z_tot*sin(x_alpha);    
    % lift L, lateral components neglected!
    L = X_tot*sin(x_alpha)-Z_tot*cos(x_alpha);
      
    
    
    %% Differential Equation
    
    f = acado.DifferentialEquation();   % system function defs
    
    f.add(dot(x_Q) ==  M_tot/Jy);   % q_dot
    f.add(dot(x_theta) ==  x_Q);    % theta_dot
    
    % option: include longitudinal stab. speeds in the controls if required
    f.add(dot(x_Vt) == (1/m)*(u_ft*cos(x_alpha)-D-m*g*sin(x_theta-x_alpha)));
    f.add(dot(x_alpha) == -u_ft*sin(x_alpha)-L+m*g*cos(x_theta-x_alpha)+m*x_Vt*x_Q);   
   
    
    %% Optimal Control Problem
    ocp = acado.OCP(t_start, t_end, N );    
    
    % the cost fn variables {x_theta, x_Vt, x_alpha, x_Q, u_el, u_ft};

    % h={cost} the LSQ-Function 
    h={x_theta, x_Q, x_alpha, x_Vt, u_el, u_ft};
    
    % r: reference comes in as a mex input vector
    % ref = [ r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];
    
    % must match number of stetes and control variables in the cost func.
    r = xRef;
    
    ocp.minimizeLSQ(Q, h, r); 
    
    ocp.subjectTo( f );                     
    ocp.subjectTo( min_el <= u_el <= max_el ); 
    
    % option: include longitudinal stab. speeds in the controls if required
    ocp.subjectTo( min_Vt <= x_Vt <= max_Vt );
    ocp.subjectTo( min_ft <= u_ft <= max_ft ); 
    ocp.subjectTo( min_alpha <= x_alpha <= max_alpha );
        
    
    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, T_s);  
    
    % algo.set('MAX_NUM_ITERATIONS', 3 ); 
    % algo.set('HESSIAN_APPROXIMATION', 'GAUSS_NEWTON' );
    
    
    %% CONTROLLER
    
    % reference trajectory
    % options: line, spline etc to link initial and final ref values
    % ref = [ startime      x0_theta  x0_Q  x0_alpha  x0_Vt  0  ft_trim
    %         startime+T_h  r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];        
     
    ref = xRefT;

    reference = acado.StaticReferenceTrajectory(ref);    
                                                        
    controller = acado.Controller( algo,reference ); 
 
    controller.init(startime, xu0);     
    controller.step(startime, xu0);


END_ACADO;    

