/*
*    This file is part of ACADO Toolkit.
*
*    ACADO Toolkit -- A Toolkit for Automatic Control and Dynamic Optimization.
*    Copyright (C) 2008-2009 by Boris Houska and Hans Joachim Ferreau, K.U.Leuven.
*    Developed within the Optimization in Engineering Center (OPTEC) under
*    supervision of Moritz Diehl. All rights reserved.
*
*    ACADO Toolkit is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Lesser General Public
*    License as published by the Free Software Foundation; either
*    version 3 of the License, or (at your option) any later version.
*
*    ACADO Toolkit is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Lesser General Public License for more details.
*
*    You should have received a copy of the GNU Lesser General Public
*    License along with ACADO Toolkit; if not, write to the Free Software
*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*
*/


/**
*    Author David Ariens
*    Date 2009-2010
*    http://www.acadotoolkit.org/matlab 
*/

#include <acado_optimal_control.hpp>
#include <acado_toolkit.hpp>
#include <acado/utils/matlab_acado_utils.hpp>

USING_NAMESPACE_ACADO


void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] ) 
 { 
 
    clearAllStaticCounters( ); 
 
    acadoPrintf("\nACADO Toolkit for Matlab - Developed by David Ariens, 2009-2010 \n"); 
    acadoPrintf("Support available at http://www.acadotoolkit.org/matlab \n \n"); 

    if (nrhs != 16){ 
      mexErrMsgTxt("This problem expects 16 right hand side argument(s) since you have defined 16 MexInput(s)");
    } 
 
    TIME autotime;
    DifferentialState x_theta;
    DifferentialState x_Q;
    DifferentialState x_alpha;
    DifferentialState x_Vt;
    Control u_el;
    Control u_ft;
    double acadoconstant1 = 2;
    double acadoconstant2 = 1.132000E+001;
    double acadoconstant3 = -2.890000E+001;
    double acadoconstant4 = -1.000000E-001;
    double acadoconstant5 = 2;
    double acadoconstant6 = 1.132000E+001;
    double acadoconstant7 = -5.230000E+000;
    double acadoconstant8 = 5.000000E-002;
    double acadoconstant9 = -9.000000E-003;
    double acadoconstant10 = 300;
    double acadoconstant11 = 1.132000E+001;
    double acadoconstant12 = 2;
    double acadoconstant13 = 1.132000E+001;
    double acadoconstant14 = 3.080000E-001;
    double acadoconstant15 = -2.100000E-002;
    double acadoconstant16 = 300;
    double acadoconstant17 = 300;
    double acadoconstant18 = 55814;
    double acadoconstant19 = 2.049036E+004;
    double acadoconstant20 = 1.570007E-003;
    double acadoconstant21 = 2.049036E+004;
    double acadoconstant22 = 6.369400E+002;
    double acadoconstant23 = 0;
    double acadoconstant24 = 5.000000E-002;
    double acadoconstant25 = 10;
    Function acadodata_f2;
    acadodata_f2 << x_theta;
    acadodata_f2 << x_Q;
    acadodata_f2 << x_alpha;
    acadodata_f2 << x_Vt;
    double acadoconstant26 = -15;
    double acadoconstant27 = 15;
    double acadoconstant28 = 300;
    double acadoconstant29 = 900;
    double acadoconstant30 = 1000;
    double acadoconstant31 = 19000;
    double acadoconstant32 = -10;
    double acadoconstant33 = 45;
    double acadoconstant34 = 5.000000E-003;
    Matrix acadodata_M1_m(readFromFile( "pitch_autopilot_data_acadodata_M1.txt" ));
    VariablesGrid acadodata_M1(acadodata_M1_m);
    Vector acadodata_v1(4);
    acadodata_v1(0) = 0;
    acadodata_v1(1) = 0;
    acadodata_v1(2) = 0;
    acadodata_v1(3) = 0;
    Vector acadodata_v2(4);
    acadodata_v2(0) = 0;
    acadodata_v2(1) = 0;
    acadodata_v2(2) = 0;
    acadodata_v2(3) = 0;
    double *mexinput0_temp = NULL; 
    if( !mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) || !(mxGetM(prhs[0])==1 && mxGetN(prhs[0])==1) ) { 
      mexErrMsgTxt("Input 0 must be a noncomplex scalar double.");
    } 
    mexinput0_temp = mxGetPr(prhs[0]); 
    double mexinput0 = *mexinput0_temp; 

    double *mexinput1_temp = NULL; 
    if( !mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) || !(mxGetM(prhs[1])==1 && mxGetN(prhs[1])==1) ) { 
      mexErrMsgTxt("Input 1 must be a noncomplex scalar double.");
    } 
    mexinput1_temp = mxGetPr(prhs[1]); 
    double mexinput1 = *mexinput1_temp; 

    double *mexinput2_temp = NULL; 
    if( !mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) || !(mxGetM(prhs[2])==1 && mxGetN(prhs[2])==1) ) { 
      mexErrMsgTxt("Input 2 must be a noncomplex scalar double.");
    } 
    mexinput2_temp = mxGetPr(prhs[2]); 
    double mexinput2 = *mexinput2_temp; 

    double *mexinput3_temp = NULL; 
    if( !mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) || !(mxGetM(prhs[3])==1 && mxGetN(prhs[3])==1) ) { 
      mexErrMsgTxt("Input 3 must be a noncomplex scalar double.");
    } 
    mexinput3_temp = mxGetPr(prhs[3]); 
    double mexinput3 = *mexinput3_temp; 

    double *mexinput4_temp = NULL; 
    if( !mxIsDouble(prhs[4]) || mxIsComplex(prhs[4]) || !(mxGetM(prhs[4])==1 && mxGetN(prhs[4])==1) ) { 
      mexErrMsgTxt("Input 4 must be a noncomplex scalar double.");
    } 
    mexinput4_temp = mxGetPr(prhs[4]); 
    double mexinput4 = *mexinput4_temp; 

    double *mexinput5_temp = NULL; 
    if( !mxIsDouble(prhs[5]) || mxIsComplex(prhs[5]) || !(mxGetM(prhs[5])==1 && mxGetN(prhs[5])==1) ) { 
      mexErrMsgTxt("Input 5 must be a noncomplex scalar double.");
    } 
    mexinput5_temp = mxGetPr(prhs[5]); 
    double mexinput5 = *mexinput5_temp; 

    double *mexinput6_temp = NULL; 
    if( !mxIsDouble(prhs[6]) || mxIsComplex(prhs[6]) || !(mxGetM(prhs[6])==1 && mxGetN(prhs[6])==1) ) { 
      mexErrMsgTxt("Input 6 must be a noncomplex scalar double.");
    } 
    mexinput6_temp = mxGetPr(prhs[6]); 
    double mexinput6 = *mexinput6_temp; 

    double *mexinput7_temp = NULL; 
    if( !mxIsDouble(prhs[7]) || mxIsComplex(prhs[7]) || !(mxGetM(prhs[7])==1 && mxGetN(prhs[7])==1) ) { 
      mexErrMsgTxt("Input 7 must be a noncomplex scalar double.");
    } 
    mexinput7_temp = mxGetPr(prhs[7]); 
    double mexinput7 = *mexinput7_temp; 

    double *mexinput8_temp = NULL; 
    if( !mxIsDouble(prhs[8]) || mxIsComplex(prhs[8]) ) { 
      mexErrMsgTxt("Input 8 must be a noncomplex double vector of dimension XxY.");
    } 
    mexinput8_temp = mxGetPr(prhs[8]); 
    Matrix mexinput8_m(mxGetM(prhs[8]), mxGetN(prhs[8]));
    for( int i=0; i<mxGetN(prhs[8]); ++i ){ 
        for( int j=0; j<mxGetM(prhs[8]); ++j ){ 
           mexinput8_m(j,i) = mexinput8_temp[i*mxGetM(prhs[8]) + j];
        } 
    } 
    VariablesGrid mexinput8(mexinput8_m); 

    double *mexinput9_temp = NULL; 
    if( !mxIsDouble(prhs[9]) || mxIsComplex(prhs[9]) || !(mxGetM(prhs[9])==1 && mxGetN(prhs[9])==1) ) { 
      mexErrMsgTxt("Input 9 must be a noncomplex scalar double.");
    } 
    mexinput9_temp = mxGetPr(prhs[9]); 
    double mexinput9 = *mexinput9_temp; 

    double *mexinput10_temp = NULL; 
    if( !mxIsDouble(prhs[10]) || mxIsComplex(prhs[10]) || !(mxGetM(prhs[10])==1 && mxGetN(prhs[10])==1) ) { 
      mexErrMsgTxt("Input 10 must be a noncomplex scalar double.");
    } 
    mexinput10_temp = mxGetPr(prhs[10]); 
    double mexinput10 = *mexinput10_temp; 

    double *mexinput11_temp = NULL; 
    if( !mxIsDouble(prhs[11]) || mxIsComplex(prhs[11]) || !(mxGetM(prhs[11])==1 && mxGetN(prhs[11])==1) ) { 
      mexErrMsgTxt("Input 11 must be a noncomplex scalar double.");
    } 
    mexinput11_temp = mxGetPr(prhs[11]); 
    double mexinput11 = *mexinput11_temp; 

    double *mexinput12_temp = NULL; 
    if( !mxIsDouble(prhs[12]) || mxIsComplex(prhs[12]) || !(mxGetM(prhs[12])==1 && mxGetN(prhs[12])==1) ) { 
      mexErrMsgTxt("Input 12 must be a noncomplex scalar double.");
    } 
    mexinput12_temp = mxGetPr(prhs[12]); 
    double mexinput12 = *mexinput12_temp; 

    double *mexinput13_temp = NULL; 
    if( !mxIsDouble(prhs[13]) || mxIsComplex(prhs[13]) || !(mxGetM(prhs[13])==1 && mxGetN(prhs[13])==1) ) { 
      mexErrMsgTxt("Input 13 must be a noncomplex scalar double.");
    } 
    mexinput13_temp = mxGetPr(prhs[13]); 
    double mexinput13 = *mexinput13_temp; 

    int mexinput14_count = 0;
    if (mxGetM(prhs[14]) == 1 && mxGetN(prhs[14]) >= 1) 
       mexinput14_count = mxGetN(prhs[14]);
    else if (mxGetM(prhs[14]) >= 1 && mxGetN(prhs[14]) == 1) 
       mexinput14_count = mxGetM(prhs[14]);
    else 
       mexErrMsgTxt("Input 14 must be a noncomplex double vector of dimension 1xY.");

    double *mexinput14_temp = NULL; 
    if( !mxIsDouble(prhs[14]) || mxIsComplex(prhs[14])) { 
      mexErrMsgTxt("Input 14 must be a noncomplex double vector of dimension 1xY.");
    } 
    mexinput14_temp = mxGetPr(prhs[14]); 
    Vector mexinput14(mexinput14_count);
    for( int i=0; i<mexinput14_count; ++i ){ 
        mexinput14(i) = mexinput14_temp[i];
    } 

    double *mexinput15_temp = NULL; 
    if( !mxIsDouble(prhs[15]) || mxIsComplex(prhs[15]) || !(mxGetM(prhs[15])==1 && mxGetN(prhs[15])==1) ) { 
      mexErrMsgTxt("Input 15 must be a noncomplex scalar double.");
    } 
    mexinput15_temp = mxGetPr(prhs[15]); 
    double mexinput15 = *mexinput15_temp; 

    DifferentialEquation acadodata_f1;
    acadodata_f1 << dot(x_Q) ==  ( ( ( (mexinput13)*(acadoconstant10) )*(acadoconstant11) )*(  acadoconstant9 +  ( acadoconstant4 +  ( ( (acadoconstant2)/( (acadoconstant1)*(x_Vt) ) )*(acadoconstant3) )*(x_Q)  )*(acadoconstant8)   +  ( ( (acadoconstant6)/( (acadoconstant5)*(x_Vt) ) )*(acadoconstant7) )*(x_Q)  ) )/(acadoconstant18) ;
    acadodata_f1 << dot(x_theta) == x_Q;
    acadodata_f1 << dot(x_Vt) ==  (acadoconstant20)*(   (u_ft)*(cos(x_alpha))  -   (  -  ( (mexinput13)*(acadoconstant16) )*( acadoconstant15 +  ( ( (acadoconstant13)/( (acadoconstant12)*(x_Vt) ) )*(acadoconstant14) )*(x_Q)  )  )*(cos(x_alpha))  -  ( ( (mexinput13)*(acadoconstant17) )*( acadoconstant4 +  ( ( (acadoconstant2)/( (acadoconstant1)*(x_Vt) ) )*(acadoconstant3) )*(x_Q)  ) )*(sin(x_alpha))    -  (acadoconstant19)*(sin( x_theta - x_alpha ))  ) ;
    acadodata_f1 << dot(x_alpha ==     (  - u_ft )*(sin(x_alpha))  -   ( ( (mexinput13)*(acadoconstant16) )*( acadoconstant15 +  ( ( (acadoconstant13)/( (acadoconstant12)*(x_Vt) ) )*(acadoconstant14) )*(x_Q)  ) )*(sin(x_alpha))  -  ( ( (mexinput13)*(acadoconstant17) )*( acadoconstant4 +  ( ( (acadoconstant2)/( (acadoconstant1)*(x_Vt) ) )*(acadoconstant3) )*(x_Q)  ) )*(cos(x_alpha))    +  (acadoconstant21)*(cos( x_theta - x_alpha ))   +  ( (acadoconstant22)*(x_Vt) )*(x_Q)  );

    OCP ocp1(acadoconstant23, acadoconstant24, acadoconstant25);
    ocp1.minimizeLSQ(acadodata_f2);
    ocp1.subjectTo(acadodata_f1);
    ocp1.subjectTo(acadoconstant26 <= u_el <= acadoconstant27);
    ocp1.subjectTo(acadoconstant28 <= x_Vt <= acadoconstant29);
    ocp1.subjectTo(acadoconstant30 <= u_ft <= acadoconstant31);
    ocp1.subjectTo(acadoconstant32 <= x_alpha <= acadoconstant33);

    RealTimeAlgorithm algo1(ocp1, acadoconstant34);

    StaticReferenceTrajectory referencetrajectory(acadodata_M1);
    Controller controller1( algo1,referencetrajectory );
    controller1.init(mexinput7, acadodata_v1);
    controller1.step(mexinput7, acadodata_v2);

    const char* outputFieldNames[] = {"U", "P"}; 
    plhs[0] = mxCreateStructMatrix( 1,1,2,outputFieldNames ); 
    mxArray *OutU = NULL;
    double  *outU = NULL;
    OutU = mxCreateDoubleMatrix( 1,controller1.getNU(),mxREAL ); 
    outU = mxGetPr( OutU );
    Vector vec_outU; 
    controller1.getU(vec_outU); 
    for( int i=0; i<vec_outU.getDim(); ++i ){ 
        outU[i] = vec_outU(i); 
    } 

    mxArray *OutP = NULL;
    double  *outP = NULL;
    OutP = mxCreateDoubleMatrix( 1,controller1.getNP(),mxREAL ); 
    outP = mxGetPr( OutP );
    Vector vec_outP; 
    controller1.getP(vec_outP); 
    for( int i=0; i<vec_outP.getDim(); ++i ){ 
        outP[i] = vec_outP(i); 
    } 

    mxSetField( plhs[0],0,"U",OutU );
    mxSetField( plhs[0],0,"P",OutP );


    clearAllStaticCounters( ); 
 
} 

