function u_el = pitch_autopilot(long_states, pitch_ref, t0)
%----------------------------------------------------------
% A wrapper for pitch_NMPC function
%----------------------------------------------------------

% extract states from func. input
% x0_alt  = 1.0; % long_states(1);
x0_theta= long_states(2);
x0_Vt   = long_states(3);
x0_alpha= long_states(4);
x0_Q    = long_states(5);
xu0 = [x0_theta, x0_Vt, x0_alpha, x0_Q, 0.0, 5000.0];

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)
% freq = 200Hz -> T_sample = 0.005 
% chosen T_horizon = 0.05 -> N = T_horizon/T_sample = 10

% T_s = 0.005;    % sampling time not accepted as acado mex input
T_h = 0.05;     % pridiction and control horizon
N = 10.0;

% reference trajectory
% options: line, spline etc to link initial and final ref values
% ref = [ startime      x0_theta  x0_Q  x0_alpha  x0_Vt  0  ft_trim
%         startime+T_h  r_theta   r_Q   r_alpha   r_Vt   0  ft_trim ];        

r_theta = pitch_ref; % pitch reference [deg]  

r_Q     = 20.0;     % pitch rate ref [deg/s]
r_alpha = 20.0;     % trim angle of attack   
r_Vt    = 350.0;    % trim velocity   
r_el    = 0.0;      % ref elevator deflection
ft_trim = 350.0;    % trim thrust 

% ref trajectory option 1: straight line link, t0 available in workspace
refT = [ t0      x0_theta  x0_Q  x0_alpha  x0_Vt  r_el  ft_trim
         t0+T_h  r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
      
% optimization reference
ref = [ r_theta   r_Q   r_alpha   r_Vt   r_el  ft_trim ];
      
% weighting matrix
Q = eye(6);
Q(1,1)= 100.0;

% run acado NMPC algorithm
  out = pitch_NMPC_RUN(t0, T_h, N, xu0, ref, refT, Q);   % 
  u_el = out.U(1);

end



