function fnc_out = roll_autopilot(lat_states, psi_ref, phi_ref, t0, p0_qbar, u0_ail, u0_rud, p0_ft, long_forces)
%----------------------------------------------------------
% A wrapper for lat_NMPC function
%----------------------------------------------------------

% extract states and parameters from func. input
x0_phi    = lat_states(1);
p0_theta  = lat_states(2);
% x0_psi    = lat_states(3);
p0_Vt     = lat_states(4);
p0_alpha  = lat_states(5);
x0_beta   = lat_states(6);
x0_P      = lat_states(7);
x0_R      = lat_states(8);

p0_X_tot = long_forces(2); 
p0_Z_tot = long_forces(3);

% initialize with current state value for continuity
u0_ail_dot=0.0;
u0_rud_dot=0.0;
% xu0 = [x0_beta, x0_phi, x0_psi, x0_P, x0_R, u0_ail, u0_ail_dot, u0_rud, u0_rud_dot]; % u0_rud is the actual deflection state, not cmd!
xu0 = [x0_beta, x0_phi, x0_P, x0_R, u0_ail, u0_ail_dot, u0_rud, u0_rud_dot]; % u0_rud is the actual deflection state, not cmd!
p0 =  [p0_qbar, p0_alpha, p0_Vt, p0_theta, p0_ft, p0_X_tot, p0_Z_tot]; % for parameter init

% optimal control parameters
% prediction horizon 
% (trade off between optimality and online computational load)
% freq = 200Hz -> T_sample = 0.005, but using 0.05 instead 
% chosen T_horizon = 0.5 -> N = T_horizon/T_sample = 10

T_s = 0.05;     % NB!!! sampling time not accepted as acado mex input
T_h = 1.0;      % pridiction and control horizon
N = T_h/T_s;    % 10.0;

% reference trajectory
% options: line, spline etc to link initial and final ref values
% ref = [ startime      p0_theta  x0_P  p0_alpha  p0_Vt  0  ft_trim
%         startime+T_h  r_P   r_R   r_phi   r_Vt   0  ft_trim ];        

d2r = pi/180.0; % degrees to radians
% r2d = 180.0/pi; % radians to degrees
r_psi = psi_ref*d2r;% heading reference [rad]
r_phi = phi_ref*d2r;% roll reference [rad] 
r_beta = 0.0;       % sideslip ref [rad]
r_P = 0.0; % roll rate reference [rad/s]
r_R = 0.0; % yaw rate reference [rad/s] 
r_rud = u0_rud;      % rudder ref
r_ail = u0_ail;      % aileron ref
% r_phi = r_P;  % [rad]; (try for unstable modes: NOT ok!) % pitcpsi_ref (ok for stable region tests) alpha assumed equal to pitch (arguement for eliminating alpha equation)   
% r_Vt = p0_Vt;       % keep speed parameter variation to a minimum
% r_qbar = p0_qbar;     % keep dynamic pressure parameter variation to a minimum

% ref trajectory option 1: straight line link, t0 available in workspace
% refT = [ t0      r_P    r_phi    r_rud    r_Vt     r_qbar
%          t0+T_h  r_P    r_phi    r_rud    r_Vt     r_qbar ];
       
% optimization reference
% ref = [ r_beta  r_phi  r_psi  r_R  r_ail  r_rud ];    % running cost reference
ref = [ r_beta  r_phi  r_P  r_R  r_ail  r_rud ];    % running cost reference
sRef= [ r_psi   r_phi ];    % terminal cost reference

% reference trajectory generation
tS1 = t0+(T_h/5.0); % 1st coincidence or intermediate time point (2*T_s?)
tS2 = t0+(T_h/2.0); % 2nd coincidence or intermediate time point (5*T_s?)
% tS1_el= t0+(T_h/10.0);% 0.05s coincidence point= time constant for elevator

% xS1_psi = x0_psi+(r_psi-x0_psi)*0.632; % assuming tS = time constant
xS1_beta = x0_beta+(r_beta-x0_beta)*0.632; % assuming tS= time constant
xS1_phi = x0_phi+(r_phi-x0_phi)*0.632; % 
% uS1_el    = u0_rud+(r_rud-u0_rud)*0.632; % 

% xS2_psi = x0_psi+(r_psi-x0_psi)*0.95; %
xS2_beta = x0_beta+(r_beta-x0_beta)*0.95; %
xS2_phi = x0_phi+(r_phi-x0_phi)*0.95; % 
% uS2_el = u0_rud+(r_rud-u0_rud)*0.95; % 

% coincidence or intermediate points within the prediction horizon
time_ref0 = [t0 tS1 tS2 t0+T_h];
% time_ref0_el = [t0 tS1_el tS2 t0+T_h];

% psi_ref0    = [x0_psi xS1_psi xS2_psi r_psi];
beta_ref0   = [x0_beta xS1_beta xS2_beta r_beta];
phi_ref0    = [p0_alpha xS1_phi xS2_phi r_phi];  
% el_ref0   = [u0_rud    uS1_el    uS2_el    r_rud];  

% ref trajectory generated using pchip 
% (Piecewise Cubic Hermite Interpolating Polynomial)
% time_ref = t0: T_s/(t0+T_h) : (t0+T_h); 
time_ref = t0: T_h/N : (t0+T_h); 
% psi_ref  = pchip(time_ref0, psi_ref0, time_ref);
beta_ref = pchip(time_ref0, beta_ref0, time_ref);
phi_ref  = pchip(time_ref0, phi_ref0, time_ref);

% other constant refs must have same length as beta_ref and phi_ref
% beta_ref   = r_beta*ones(1,T_h/T_s+1); 
% phi_ref   = r_phi*ones(1,T_h/T_s+1); 
rud_refC   = r_rud*ones(1,T_h/T_s+1); 
ail_refC   = r_ail*ones(1,T_h/T_s+1);
P_ref   = r_P*ones(1,T_h/T_s+1);
R_ref   = r_R*ones(1,T_h/T_s+1);
% Vt_ref   = r_Vt*ones(1,T_h/T_s+1);
% qbar_ref = r_qbar*ones(1,T_h/T_s+1);

% the reference trajectory matrix for acado mex interface 
% refT = [ t0      p0_theta  p0_alpha   r_rud   r_ail    r_Vt     r_qbar
%          tS      xS_theta  xS_alpha   r_rud   r_ail    r_Vt     r_qbar
%          t0+T_h  r_P   r_phi    r_rud   r_ail    r_Vt     r_qbar ];

% refT = [time_ref', beta_ref', phi_ref', psi_ref', R_ref', ail_refC', rud_refC'];
refT = [time_ref', beta_ref', phi_ref', P_ref', R_ref', ail_refC', rud_refC'];

% weighting matrices 
Q_R = eye(6);          % Q and R weight matrices
Q_R(1,1)= 1.0;        % beta, sideslip
Q_R(2,2)= 10.0;        % phi, roll
% Q_R(2,2)= 1000.0;      % psi, heading
Q_R(3,3)= 1.0;        % P, roll rate
Q_R(4,4)= 1.0;        % R, yaw rate
Q_R(5,5)= 0.05;       % uC_ail, aileron deflection cmd
Q_R(6,6)= 0.1;       % uC_rud,rudder cmd

% terminal weight
S = eye(2);     % P weight matrix
S(1,1)= 1000.0;  % theta
S(2,2)= 100.0;   % alpha

% input position limits
min_ail = -15.0;
max_ail = 15.0;  
min_rud = -15.0;
max_rud = 15.0;  
  
% run acado NMPC algorithm
la_out = roll_NMPC_RUN(t0, T_h, N, xu0, ref, refT, Q_R, p0, S, sRef, min_ail,... 
      max_ail, min_rud, max_rud, p0_qbar, p0_alpha, p0_Vt,p0_theta, p0_ft,...
      p0_X_tot, p0_Z_tot);
    
  u_ail = la_out.U(1);
  u_rud = la_out.U(2);
  p_qbar = la_out.P(1);
  p_alpha = la_out.P(2);
  p_Vt = la_out.P(3);
  p_theta = la_out.P(4);
  p_ft = la_out.P(5);
  p_X_tot = la_out.P(6);
  p_Z_tot = la_out.P(7);
  
  fnc_out = [u_ail, u_rud, p_qbar, p_alpha, p_Vt, p_theta, p_ft, p_X_tot, p_Z_tot]';

end



